import os
import subprocess

ROOT = os.path.realpath(os.path.dirname(__file__) + '/..') + '/'
os.chdir(ROOT+'tests')


def test_hitemp_single_zip(capfd):
    subprocess.call('repack hitemp_repack_single_zip.cfg'.split())
    capfd = capfd.readouterr()
    assert """Reading: 'data/02_03750-04000_HITEMP2010.zip'.
  Flagging lines at  500 K:
  Compression rate:       76.11%,     51,071/   213,769 lines.
  Flagging lines at  700 K:
  Compression rate:       67.97%,     68,468/   213,769 lines.
  Total compression rate: 66.26%,     72,118/   213,769 lines.

With a threshold strength factor of 0.01,
kept a total of 72,118 line transitions out of 213,769 lines.

Successfully rewriten hitran line-transition info into:
  'CO2_hitran_2.5-2.6um_500-700K_lbl.dat' and
  'CO2_hitran_2.5-2.6um_500-700K_continuum.dat'.""" in capfd.out


def test_hitemp_single_unzip(capfd):
    # Unzip files before repacking:
    subprocess.call('unzip data/02_03750-04000_HITEMP2010.zip -d data/'.split())
    subprocess.call('repack hitemp_repack_single_unzip.cfg'.split())
    capfd = capfd.readouterr()
    assert """Reading: 'data/02_3750-4000_HITEMP2010.par'.
  Flagging lines at  500 K:
  Compression rate:       76.11%,     51,071/   213,769 lines.
  Flagging lines at  700 K:
  Compression rate:       67.97%,     68,468/   213,769 lines.
  Total compression rate: 66.26%,     72,118/   213,769 lines.

With a threshold strength factor of 0.01,
kept a total of 72,118 line transitions out of 213,769 lines.

Successfully rewriten hitran line-transition info into:
  'CO2_hitran_2.5-2.6um_500-700K_lbl.dat' and
  'CO2_hitran_2.5-2.6um_500-700K_continuum.dat'.""" in capfd.out
    # Teardown:
    os.remove('data/02_3750-4000_HITEMP2010.par')


def test_hitemp_two_files(capfd):
    subprocess.call(['repack', 'hitemp_repack_two.cfg'])
    capfd = capfd.readouterr()
    assert """Reading: 'data/02_03750-04000_HITEMP2010.zip'.
  Flagging lines at  500 K:
  Compression rate:       76.11%,     51,071/   213,769 lines.
  Flagging lines at  700 K:
  Compression rate:       67.97%,     68,468/   213,769 lines.
  Total compression rate: 66.26%,     72,118/   213,769 lines.

Reading: 'data/02_04000-04500_HITEMP2010.zip'.
  Flagging lines at  500 K:
  Compression rate:       46.12%,     74,492/   138,258 lines.
  Flagging lines at  700 K:
  Compression rate:       18.91%,    112,116/   138,258 lines.
  Total compression rate: 18.65%,    112,469/   138,258 lines.

With a threshold strength factor of 0.01,
kept a total of 184,587 line transitions out of 352,027 lines.

Successfully rewriten hitran line-transition info into:
  'CO2_hitran_2.2-2.6um_500-700K_lbl.dat' and
  'CO2_hitran_2.2-2.6um_500-700K_continuum.dat'.""" in capfd.out


def test_hitemp_single_chunks(capfd):
    subprocess.call('repack hitemp_repack_single_chunks.cfg'.split())
    capfd = capfd.readouterr()
    assert """Reading: 'data/02_03750-04000_HITEMP2010.zip'.
  Flagging lines at  500 K (chunk 1/3):
  Compression rate:       83.40%,     11,826/    71,256 lines.
  Flagging lines at  700 K:
  Compression rate:       78.32%,     15,451/    71,256 lines.
  Total compression rate: 76.31%,     16,883/    71,256 lines.

  Flagging lines at  500 K (chunk 2/3):
  Compression rate:       77.27%,     16,199/    71,256 lines.
  Flagging lines at  700 K:
  Compression rate:       68.75%,     22,271/    71,256 lines.
  Total compression rate: 66.94%,     23,554/    71,256 lines.

  Flagging lines at  500 K (chunk 3/3):
  Compression rate:       67.64%,     23,057/    71,257 lines.
  Flagging lines at  700 K:
  Compression rate:       56.82%,     30,771/    71,257 lines.
  Total compression rate: 55.50%,     31,706/    71,257 lines.

With a threshold strength factor of 0.01,
kept a total of 72,143 line transitions out of 213,769 lines.

Successfully rewriten hitran line-transition info into:
  'CO2_hitran_2.5-2.6um_500-700K_lbl.dat' and
  'CO2_hitran_2.5-2.6um_500-700K_continuum.dat'.""" in capfd.out




# Copyright (c) 2017-2021 Patricio Cubillos and contributors.
# repack is open-source software under the MIT license (see LICENSE).

# repack version:
repack_VER = 1  # Major version
repack_MIN = 4  # Minor version
repack_REV = '5a1'  # Revision

__version__ = f'{repack_VER}.{repack_MIN}.{repack_REV}'



#! /usr/bin/env python

# Copyright (c) 2017-2021 Patricio Cubillos and contributors.
# repack is open-source software under the MIT license (see LICENSE).

import sys
import time
import warnings
from datetime import date

import repack


banner = ':'*70
error_message = """\
  Error: Wrong usage.  For repacking:
      repack config_file.cfg
  For sorting (of MARVELized data):
      repack -sort config_file.cfg"""

def main():
    warnings.simplefilter("ignore", RuntimeWarning)
    if len(sys.argv) == 3:
        if sys.argv[1] != '-sort':
            print(f"\n{banner}\n{error_message}.\n{banner}\n")
            sys.exit(0)

    elif len(sys.argv) != 2:
        print(f"\n{banner}\n  Error: Wrong usage.\n{banner}\n")
        sys.exit(0)

    print(f"\n{banner}\n"
           "  repack: line-transition data compression.\n"
          f"  Version {repack.__version__}.\n"
          f"  Copyright (c) 2017-{date.today().year} Patricio Cubillos.\n"
           "  repack is open-source software under the MIT license.\n"
          f"{banner}\n\n")

    if len(sys.argv) == 2 and sys.argv[1] == '-v':
        return

    print(f"Start: {time.ctime()}")
    if len(sys.argv) == 3:
        repack.sort(sys.argv[2])
    else:
        repack.repack(sys.argv[1])
    print(f"End: {time.ctime()}")


if __name__ == "__main__":
    main()



import os
import subprocess
import pytest
import repack

ROOT = os.path.realpath(os.path.dirname(__file__) + '/..') + '/'
os.chdir(ROOT+'tests')


def test_exomol_sort():
    if os.path.exists('data/1H2-16O__POKAZATEL__34800-34900.trans.sort.bz2'):
        os.remove('data/1H2-16O__POKAZATEL__34800-34900.trans.sort.bz2')
    subprocess.call('repack -sort exomol_sort.cfg'.split())
    assert os.path.exists('data/1H2-16O__POKAZATEL__34800-34900.trans.sort.bz2')


def test_exomol_repack_sorted(capfd):
    subprocess.call('repack exomol_sort.cfg'.split())
    capfd = capfd.readouterr()
    assert """Unzipping: 'data/1H2-16O__POKAZATEL__34800-34900.trans.sort.bz2'.
Reading: 'data/1H2-16O__POKAZATEL__34800-34900.trans.sort.bz2'.
  Flagging lines at  500 K:
  Compression rate:       98.46%,      2,375/   154,580 lines.
  Flagging lines at 1000 K:
  Compression rate:       97.35%,      4,095/   154,580 lines.
  Total compression rate: 97.11%,      4,465/   154,580 lines.

With a threshold strength factor of 0.1,
kept a total of 4,465 line transitions out of 154,580 lines.

Successfully rewriten exomol line-transition info into:
  'H2O_exomol_0.26um_500-1000K_lbl.dat'.""" in capfd.out



import os
import subprocess
import pytest

ROOT = os.path.realpath(os.path.dirname(__file__) + '/..') + '/'
os.chdir(ROOT+'tests')


@pytest.mark.parametrize('missfile',
    ['exomol_repack_missing1.cfg',
     'exomol_repack_missing2.cfg'])
def test_missing(capfd, missfile):
    subprocess.call(['repack', missfile])
    capfd = capfd.readouterr()
    assert """File(s) not Found Error: These files are missing:
  data/14N-1H3__MiSSinG__00200-00300.trans.bz2""" in capfd.out


def test_exomol_single(capfd):
    subprocess.call('repack exomol_repack_single.cfg'.split())
    capfd = capfd.readouterr()
    assert """Unzipping: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
Reading: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
  Flagging lines at  500 K:
  Compression rate:       74.01%,    269,648/ 1,037,545 lines.
  Flagging lines at  700 K:
  Compression rate:       72.35%,    286,925/ 1,037,545 lines.
  Total compression rate: 70.75%,    303,456/ 1,037,545 lines.

With a threshold strength factor of 0.01,
kept a total of 303,456 line transitions out of 1,037,545 lines.

Successfully rewriten exomol line-transition info into:
  'NH3_exomol_050-100um_500-700K_lbl.dat' and
  'NH3_exomol_050-100um_500-700K_continuum.dat'.""" in capfd.out


def test_exomol_two_files(capfd):
    subprocess.call('repack exomol_repack_two_files.cfg'.split())
    capfd = capfd.readouterr()
    assert """Unzipping: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
Unzipping: 'data/14N-1H3__BYTe__00200-00300.trans.bz2'.
Reading: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
  Flagging lines at  500 K:
  Compression rate:       74.01%,    269,648/ 1,037,545 lines.
  Flagging lines at  700 K:
  Compression rate:       72.35%,    286,925/ 1,037,545 lines.
  Total compression rate: 70.75%,    303,456/ 1,037,545 lines.

Reading: 'data/14N-1H3__BYTe__00200-00300.trans.bz2'.
  Flagging lines at  500 K:
  Compression rate:       83.19%,    178,663/ 1,062,896 lines.
  Flagging lines at  700 K:
  Compression rate:       81.90%,    192,354/ 1,062,896 lines.
  Total compression rate: 80.69%,    205,243/ 1,062,896 lines.

With a threshold strength factor of 0.01,
kept a total of 508,699 line transitions out of 2,100,441 lines.

Successfully rewriten exomol line-transition info into:
  'NH3_exomol_033-100um_500-700K_lbl.dat' and
  'NH3_exomol_033-100um_500-700K_continuum.dat'.""" in capfd.out


def test_exomol_two_isotopes(capfd):
    subprocess.call('repack exomol_repack_two_isotopes.cfg'.split())
    capfd = capfd.readouterr()
    assert """Unzipping: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
Unzipping: 'data/15N-1H3__BYTe-15__00100-00200.trans.bz2'.
Reading: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
Reading: 'data/15N-1H3__BYTe-15__00100-00200.trans.bz2'.
  Flagging lines at  500 K:
  Compression rate:       76.73%,    282,278/ 1,212,878 lines.
  Flagging lines at  700 K:
  Compression rate:       75.20%,    300,854/ 1,212,878 lines.
  Total compression rate: 73.40%,    322,684/ 1,212,878 lines.

With a threshold strength factor of 0.01,
kept a total of 322,684 line transitions out of 1,212,878 lines.

Successfully rewriten exomol line-transition info into:
  'NH3_exomol_050-100um_500-700K_lbl.dat' and
  'NH3_exomol_050-100um_500-700K_continuum.dat'.""" in capfd.out


def test_exomol_two_files_two_iso(capfd):
    subprocess.call('repack exomol_repack_two_two.cfg'.split())
    capfd = capfd.readouterr()
    assert """Unzipping: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
Unzipping: 'data/15N-1H3__BYTe-15__00100-00200.trans.bz2'.
Unzipping: 'data/14N-1H3__BYTe__00200-00300.trans.bz2'.
Unzipping: 'data/15N-1H3__BYTe-15__00200-00300.trans.bz2'.
Reading: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
Reading: 'data/15N-1H3__BYTe-15__00100-00200.trans.bz2'.
  Flagging lines at  500 K:
  Compression rate:       76.73%,    282,278/ 1,212,878 lines.
  Flagging lines at  700 K:
  Compression rate:       75.20%,    300,854/ 1,212,878 lines.
  Total compression rate: 73.40%,    322,684/ 1,212,878 lines.

Reading: 'data/14N-1H3__BYTe__00200-00300.trans.bz2'.
Reading: 'data/15N-1H3__BYTe-15__00200-00300.trans.bz2'.
  Flagging lines at  500 K:
  Compression rate:       85.06%,    184,349/ 1,234,214 lines.
  Flagging lines at  700 K:
  Compression rate:       83.84%,    199,462/ 1,234,214 lines.
  Total compression rate: 82.57%,    215,138/ 1,234,214 lines.

With a threshold strength factor of 0.01,
kept a total of 537,822 line transitions out of 2,447,092 lines.

Successfully rewriten exomol line-transition info into:
  'NH3_exomol_033-100um_500-700K_lbl.dat' and
  'NH3_exomol_033-100um_500-700K_continuum.dat'.""" in capfd.out


def test_exomol_single_unzip(capfd):
    # Unzip files before repacking:
    subprocess.call(['bzip2','-dk','data/14N-1H3__BYTe__00100-00200.trans.bz2'])
    subprocess.call('bzip2 -dk data/14N-1H3__BYTe.states.bz2'.split())
    subprocess.call('repack exomol_repack_single_unzip.cfg'.split())
    capfd = capfd.readouterr()
    assert """Reading: 'data/14N-1H3__BYTe__00100-00200.trans'.
  Flagging lines at  500 K:
  Compression rate:       74.01%,    269,648/ 1,037,545 lines.
  Flagging lines at  700 K:
  Compression rate:       72.35%,    286,925/ 1,037,545 lines.
  Total compression rate: 70.75%,    303,456/ 1,037,545 lines.

With a threshold strength factor of 0.01,
kept a total of 303,456 line transitions out of 1,037,545 lines.

Successfully rewriten exomol line-transition info into:
  'NH3_exomol_050-100um_500-700K_lbl.dat' and
  'NH3_exomol_050-100um_500-700K_continuum.dat'.""" in capfd.out
    os.remove('data/14N-1H3__BYTe__00100-00200.trans')
    os.remove('data/14N-1H3__BYTe.states')


def test_exomol_single_chunks(capfd):
    subprocess.call('repack exomol_repack_single_chunks.cfg'.split())
    capfd = capfd.readouterr()
    assert """Unzipping: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
Reading: 'data/14N-1H3__BYTe__00100-00200.trans.bz2'.
  Flagging lines at  500 K (chunk 1/2):
  Compression rate:       70.54%,    152,852/   518,772 lines.
  Flagging lines at  700 K:
  Compression rate:       68.85%,    161,620/   518,772 lines.
  Total compression rate: 67.12%,    170,554/   518,772 lines.

  Flagging lines at  500 K (chunk 2/2):
  Compression rate:       77.49%,    116,797/   518,773 lines.
  Flagging lines at  700 K:
  Compression rate:       75.85%,    125,306/   518,773 lines.
  Total compression rate: 74.38%,    132,903/   518,773 lines.

With a threshold strength factor of 0.01,
kept a total of 303,457 line transitions out of 1,037,545 lines.

Successfully rewriten exomol line-transition info into:
  'NH3_exomol_050-100um_500-700K_lbl.dat' and
  'NH3_exomol_050-100um_500-700K_continuum.dat'.""" in capfd.out




# Copyright (c) 2017-2021 Patricio Cubillos and contributors.
# repack is open-source software under the MIT license (see LICENSE).

import os
import re
import sys
import setuptools
from setuptools import setup, Extension
from numpy import get_include

sys.path.append(os.path.join(os.path.dirname(__file__), 'repack'))
from VERSION import __version__


srcdir = './repack/utils/'  # C-code source folder

files = os.listdir(srcdir)
files = list(filter(lambda x: re.search('.+[.]c$', x), files))
files = list(filter(lambda x: not re.search('[.#].+[.]c$', x), files))

inc = [get_include()]
eca = ['-ffast-math']
ela = []

extensions = [
    Extension(
        'repack.utils.' + cfile.rstrip('.c'),
        sources=[f'{srcdir}{cfile}'],
        include_dirs=inc,
        extra_compile_args=eca,
        extra_link_args=ela,
        )
    for cfile in files
    ]

with open('README.md', 'r') as f:
    readme = f.read()


setup(
    name = 'lbl-repack',
    version = __version__,
    author = 'Patricio Cubillos',
    author_email = 'patricio.cubillos@oeaw.ac.at',
    url = 'https://github.com/pcubillos/repack',
    packages = setuptools.find_packages(),
    install_requires = [
        'numpy>=1.13.3',
        'scipy>=0.17.1',
        ],
    include_package_data = True,
    license = 'MIT',
    description = 'A line-transition data compression package.',
    long_description = readme,
    long_description_content_type = 'text/markdown',
    include_dirs = inc,
    entry_points = {'console_scripts': ['repack = repack.__main__:main']},
    ext_modules = extensions,
    )



# Copyright (c) 2017-2021 Patricio Cubillos and contributors.
# repack is open-source software under the MIT license (see LICENSE).

__all__ = [
    'constants',
    'utils',
    ]

from . import constants
from . import utils
from .pack import *

from .VERSION import __version__

__all__ += pack.__all__


# Clean up top-level namespace--delete everything that isn't in __all__
# or is a magic attribute, and that isn't a submodule of this package
for varname in dir():
    if not ((varname.startswith('__') and varname.endswith('__')) or
            varname in __all__):
        del locals()[varname]
del(varname)
